/**
 * @file m13_ra6m3_ek_led.c
 * @brief M13-RA6M3-EK led BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "r_adc.h"
#include "m13_ra6m3_ek_adc.h"
#include "m13_ra6m3_ek_pinout.h"
#include "m13_ra6m3_ek_status.h"
#include "m13_ra6m3_ek_vcom.h"

// Global variables
uint16_t adc_data = 0;

adc_instance_ctrl_t g_adc_ctrl;

// ADC extended configuration for UARTonSCI HAL driver
const adc_extended_cfg_t g_adc_cfg_extend =
{
     .add_average_count     = ADC_ADD_OFF,
     .clearing              = ADC_CLEAR_AFTER_READ_OFF,
     .trigger_group_b       = ADC_TRIGGER_SYNC_ELC,
     .double_trigger_mode   = ADC_DOUBLE_TRIGGER_DISABLED,
     .adc_vref_control      = ADC_VREF_CONTROL_VREFH,
};

// ADC interface configuration
const adc_cfg_t g_adc_cfg =
{
     .unit           = 0,
     .mode           = ADC_MODE_SINGLE_SCAN,
     .resolution     = ADC_RESOLUTION_12_BIT,
     .alignment      = (adc_alignment_t) ADC_ALIGNMENT_RIGHT,
     .trigger        = ADC_TRIGGER_SOFTWARE,
     .p_callback     = NULL,
     .p_context      = NULL,
     .p_extend       = &g_adc_cfg_extend,
     .scan_end_irq   = FSP_INVALID_VECTOR,
     .scan_end_ipl   = (BSP_IRQ_DISABLED),
     .scan_end_b_irq = FSP_INVALID_VECTOR,
     .scan_end_b_ipl = (BSP_IRQ_DISABLED),
};

// ADC channel configuration
adc_channel_cfg_t g_adc_channel_cfg =
{
     .scan_mask          = BSP_ADC_MASK_OFF | 0,
     .scan_mask_group_b  = 0,
     .priority_group_a   = ADC_GROUP_A_PRIORITY_OFF,
     .add_mask           = 0,
     .sample_hold_mask   = 0,
     .sample_hold_states = 24,
};

// ADC interface module
const adc_instance_t g_adc =
{
     .p_ctrl        = &g_adc_ctrl,
     .p_cfg         = &g_adc_cfg,
     .p_channel_cfg = &g_adc_channel_cfg,
     .p_api         = &g_adc_on_adc,
};

/**
 * @name       bsp_adc_io_init
 * @brief      Initialize ADC pins
 * @return     None
 **/
void bsp_adc_io_init (void)
{
    ioport_cfg_t adc_pin_cfg;

    // Load pin configuration from pin table
    adc_pin_cfg.number_of_pins = sizeof(m13_ra6m3_ek_adc_pin_data)/sizeof(ioport_pin_cfg_t);
    adc_pin_cfg.p_pin_cfg_data = &m13_ra6m3_ek_adc_pin_data[0];
}

/**
 * @name       bsp_adc_init
 * @brief      Configure ADC channel
 * @param[in]  adc_channel : adc channel
 * @return     Error code (see #ErrorCode enumeration)
 **/
bsp_status_t bsp_adc_init (bsp_adc_channel_t adc_channel)
{
    fsp_err_t error = FSP_SUCCESS;

    // Activate adc channel according to the requested adc mask
    g_adc_channel_cfg.scan_mask = ( 1 << adc_channel | 0);

    // Open/Initialize ADC module
    error = R_ADC_Open(&g_adc_ctrl, &g_adc_cfg);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_OPEN_FAILURE;
    }

    // Configures the ADC scan parameters
    error = R_ADC_ScanCfg(&g_adc_ctrl, &g_adc_channel_cfg);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_OPEN_FAILURE;
    }

    return BSP_STATUS_OK;
}

/**
 * @name       bsp_adc_get_level
 * @brief      Start ADC Scan and Read ADC value
 * @param[in]  adc_channel : adc channel
 * @return     value : ADC value
 **/
uint16_t bsp_adc_get_level (bsp_adc_channel_t adc_channel)
{

    uint16_t value;

    // Select the adc channel to scan
    //g_adc_ctrl.scan_mask =  ( 1 << adc_channel ) | 0 ;
    g_adc_channel_cfg.scan_mask = ( 1 << adc_channel ) | 0;

    // Start the ADC scan
    R_ADC_ScanStart(&g_adc_ctrl);

    // Read the adc value
    R_ADC_Read(&g_adc_ctrl, adc_channel, &value);

    // Delay of 10ms after reading
    vTaskDelay (10);

    return value;
}

/**
 * @name       bsp_adc_deinit
 * @brief      Stop ADC scan
 * @param[in]  adc_channel : adc channel
 * @return     None
 **/
bsp_status_t bsp_adc_deinit (bsp_adc_channel_t adc_channel)
{
    fsp_err_t error = FSP_SUCCESS;

    // Select the adc channel to stop
    g_adc_ctrl.scan_mask = ( ( 1 << adc_channel ) | 0 );

    // Stop the ADC scan
    error = R_ADC_ScanStop(&g_adc_ctrl);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_OPEN_FAILURE;
    }

    return BSP_STATUS_OK;
}

